@@ -53,7 +53,7 @@ typedef enum
 {
     LOK_SELTYPE_NONE,
     LOK_SELTYPE_TEXT,
-    LOK_SELTYPE_LARGE_TEXT,
+    LOK_SELTYPE_LARGE_TEXT, // unused (same as LOK_SELTYPE_COMPLEX)
     LOK_SELTYPE_COMPLEX
 }
 LibreOfficeKitSelectionType;
@@ -131,7 +131,12 @@ typedef enum
     /**
      * The size and/or the position of the visible cursor changed.
      *
-     * Rectangle format is the same as LOK_CALLBACK_INVALIDATE_TILES.
+     * Old format is the same as LOK_CALLBACK_INVALIDATE_TILES.
+     * New format is a JSON with 3 elements the 'viewId' element represented by
+     * an integer value, a 'rectangle' element in the format "x, y, width, height",
+     * and a 'misspelledWord' element represented by an integer value: '1' when
+     * a misspelled word is at the cursor position, '0' when the word is
+     * not misspelled.
      */
     LOK_CALLBACK_INVALIDATE_VISIBLE_CURSOR = 1,
     /**
@@ -211,7 +216,7 @@ typedef enum
     LOK_CALLBACK_GRAPHIC_SELECTION = 6,
 
     /**
-     * User clicked on an hyperlink that should be handled by other
+     * User clicked on a hyperlink that should be handled by other
      * applications accordingly.
      */
     LOK_CALLBACK_HYPERLINK_CLICKED = 7,
@@ -366,7 +371,7 @@ typedef enum
      * {
      *     "classification": "error" | "warning" | "info"
      *     "kind": "network" etc.
-     *     "code": a structured 32-bit error code, the ErrCode from LibreOffice's <tools/errcode.hxx>
+     *     "code": a structured 32-bit error code, the ErrCode from LibreOffice's <comphelper/errcode.hxx>
      *     "message": freeform description
      * }
      */
@@ -609,6 +614,7 @@ typedef enum
      *  - "dialog" - window is a dialog
      *  - "child" - window is a floating window (combo boxes, etc.)
      *  - "deck" - window is a docked/floating deck (i.e. the sidebar)
+     *  - "tooltip" - window is a tooltip popup
      *
      * "action" can take following values:
      * - "created" - window is created in the backend, client can render it now
@@ -638,10 +644,9 @@ typedef enum
      * Notification that the clipboard contents have changed.
      * Typically fired in response to copying to clipboard.
      *
-     * The payload currently is empty and it's up to the
-     * client to get the contents, if necessary. However,
-     * in the future the contents might be included for
-     * convenience.
+     * Payload is optional. When payload is empty, Online gets string from selected text.
+     * Payload format is JSON.
+     * Example: { "mimeType": "text/plain", "content": "some content" }
      */
     LOK_CALLBACK_CLIPBOARD_CHANGED = 38,
 
@@ -709,6 +714,191 @@ typedef enum
      * the description.
      */
     LOK_CALLBACK_JSDIALOG = 46,
+
+    /**
+     * Send the list of functions whose name starts with the characters entered
+     * by the user in the formula input bar.
+     */
+    LOK_CALLBACK_CALC_FUNCTION_LIST = 47,
+
+    /**
+     * Sends the tab stop list for the current of the current cursor position.
+     */
+    LOK_CALLBACK_TAB_STOP_LIST = 48,
+
+    /**
+     * Sends all information for displaying form field button for a text based field.
+     *
+     * It contains the position where the frame with the button should be displayed and
+     * also contains all information that the popup window needs.
+     *
+     * The payload example:
+     * {
+     *      "action": "show",
+     *      "type": "drop-down",
+     *      "textArea": "1418, 3906, 3111, 919",
+     *      "params": {
+     *           "items": ["January", "February", "July"],
+     *           "selected": "2",
+     *           "placeholder": "No items specified"
+     *      }
+     * }
+     *
+     * or
+     * {
+     *      "action": "hide",
+     *      "type": "drop-down"
+     * }
+     */
+    LOK_CALLBACK_FORM_FIELD_BUTTON = 49,
+
+    /**
+     * This is Calc specific. Indicates that some or all of the current sheet's
+     * geometry data has changed. Clients must request a full or partial sheet
+     * geometry data set.
+     *
+     * The payload specifies what part of the sheet geometry data has changed.
+     * The payload format is:
+     * 'all|rows|columns [sizes [hidden [filtered [groups]]]]'
+     *
+     * For example, the payload 'rows sizes groups' indicates that the row heights
+     * and row-groups data have changed.
+     */
+    LOK_CALLBACK_INVALIDATE_SHEET_GEOMETRY = 50,
+
+    /**
+     * When for the current cell is defined an input help text.
+     *
+     * The payload format is JSON: { "title": "title text", "content": "content text" }
+     */
+    LOK_CALLBACK_VALIDITY_INPUT_HELP = 51,
+
+    /**
+     * This is currently Calc only. Indicates the document background
+     * color in the payload as a RGB hex string (RRGGBB).
+     */
+    LOK_CALLBACK_DOCUMENT_BACKGROUND_COLOR = 52,
+
+    /**
+     * When a user tries to use command which is restricted for that user
+     */
+    LOK_COMMAND_BLOCKED = 53,
+
+    /**
+     * The position of the cell cursor jumped to.
+     *
+     * Payload format: "x, y, width, height, column, row", where the first
+     * 4 numbers are document coordinates, in twips, and the last 2 are table
+     * coordinates starting from 0.
+     * When the cursor is not shown the payload format is the "EMPTY" string.
+     *
+     * Rectangle format is the same as LOK_CALLBACK_INVALIDATE_TILES.
+     */
+    LOK_CALLBACK_SC_FOLLOW_JUMP = 54,
+
+    /**
+     * Sends all information for displaying metadata for a text based content control.
+     *
+     * Examples:
+     * Entered a rich text content control:
+     * {
+     *     "action": "show",
+     *     "alias": "my alias", // omitted if empty
+     *     "rectangles": "1418, 1694, 720, 551; 10291, 1418, 1099, 275"
+     * }
+     *
+     * Left a rich text content control:
+     * {
+     *     "action": "hide"
+     * }
+     *
+     * Entered a dropdown content control:
+     * {
+     *     "action": "show",
+     *     "rectangles": "...",
+     *     "items": ["red", "green", "blue"]
+     * }
+     *
+     * Clicked on a picture content control's placeholder:
+     * {
+     *     "action": "change-picture"
+     * }
+     *
+     * Entered a date content control:
+     * {
+     *     "action": "show",
+     *     "rectangles": "...",
+     *     "date": "true"
+     * }
+     */
+    LOK_CALLBACK_CONTENT_CONTROL = 55,
+
+    /**
+     * This is Calc specific. The payload contains print ranges of all
+     * sheets in the document.
+     *
+     * Payload example:
+     * {
+     *     "printranges" : [
+     *         {
+     *             "sheet": 0,
+     *             "ranges": [
+     *                 [0, 0, 4, 5],
+     *                 [5, 100, 8, 150]
+     *             ]
+     *         },
+     *         {
+     *             "sheet": 3,
+     *             "ranges": [
+     *                 [1, 0, 6, 10],
+     *                 [3, 200, 6, 230]
+     *             ]
+     *         }
+     *     ]
+     * }
+     *
+     * The format of the inner "ranges" array for each sheet is
+     * [<startColumn>, <startRow>, <endColumn>, <endRow>]
+     */
+    LOK_CALLBACK_PRINT_RANGES = 56,
+
+    /**
+     * Informs the LibreOfficeKit client that a font specified in the
+     * document is missing.
+     *
+     * This callback is emitted right after the document has been loaded.
+     *
+     * Payload example:
+     * {
+     *     "fontsmissing": [
+     *         "Some Random Font",
+     *         "Another Font"
+     *     ]
+     * }
+     *
+     * The names are those of the font family. Sadly it is currently
+     * not possible to know the name of the font style that is
+     * missing.
+     *
+     */
+    LOK_CALLBACK_FONTS_MISSING = 57,
+
+    /**
+     * Insertion, removal, movement, and selection of a media shape.
+     * The payload is a json with the relevant details.
+     *
+     *      {
+     *          "action": "insert",
+     *          "id": 123456,
+     *          "url": "file:// ..."
+     *          "x": ...,
+     *          "y": ...,
+     *      }
+     *
+     *      where the "svg" property is a string containing an svg document
+     *      which is a representation of the pie segment.
+     */
+    LOK_CALLBACK_MEDIA_SHAPE = 58,
 }
 LibreOfficeKitCallbackType;
 
@@ -813,6 +1003,8 @@ static inline const char* lokCallbackTypeToString(int nType)
         return "LOK_CALLBACK_WINDOW";
     case LOK_CALLBACK_VALIDITY_LIST_BUTTON:
         return "LOK_CALLBACK_VALIDITY_LIST_BUTTON";
+    case LOK_CALLBACK_VALIDITY_INPUT_HELP:
+        return "LOK_CALLBACK_VALIDITY_INPUT_HELP";
     case LOK_CALLBACK_CLIPBOARD_CHANGED:
         return "LOK_CALLBACK_CLIPBOARD_CHANGED";
     case LOK_CALLBACK_CONTEXT_CHANGED:
@@ -831,6 +1023,28 @@ static inline const char* lokCallbackTypeToString(int nType)
         return "LOK_CALLBACK_REFERENCE_MARKS";
     case LOK_CALLBACK_JSDIALOG:
         return "LOK_CALLBACK_JSDIALOG";
+    case LOK_CALLBACK_CALC_FUNCTION_LIST:
+        return "LOK_CALLBACK_CALC_FUNCTION_LIST";
+    case LOK_CALLBACK_TAB_STOP_LIST:
+        return "LOK_CALLBACK_TAB_STOP_LIST";
+    case LOK_CALLBACK_FORM_FIELD_BUTTON:
+        return "LOK_CALLBACK_FORM_FIELD_BUTTON";
+    case LOK_CALLBACK_INVALIDATE_SHEET_GEOMETRY:
+        return "LOK_CALLBACK_INVALIDATE_SHEET_GEOMETRY";
+    case LOK_CALLBACK_DOCUMENT_BACKGROUND_COLOR:
+        return "LOK_CALLBACK_DOCUMENT_BACKGROUND_COLOR";
+    case LOK_COMMAND_BLOCKED:
+        return "LOK_COMMAND_BLOCKED";
+    case LOK_CALLBACK_SC_FOLLOW_JUMP:
+        return "LOK_CALLBACK_SC_FOLLOW_JUMP";
+    case LOK_CALLBACK_CONTENT_CONTROL:
+        return "LOK_CALLBACK_CONTENT_CONTROL";
+    case LOK_CALLBACK_PRINT_RANGES:
+        return "LOK_CALLBACK_PRINT_RANGES";
+    case LOK_CALLBACK_FONTS_MISSING:
+        return "LOK_CALLBACK_FONTS_MISSING";
+    case LOK_CALLBACK_MEDIA_SHAPE:
+        return "LOK_CALLBACK_MEDIA_SHAPE";
     }
 
     assert(!"Unknown LibreOfficeKitCallbackType type.");
@@ -839,11 +1053,11 @@ static inline const char* lokCallbackTypeToString(int nType)
 
 typedef enum
 {
-    /// A pressed gesture has started.
+    /// A mouse button has been pressed down.
     LOK_MOUSEEVENT_MOUSEBUTTONDOWN,
-    /// A pressed gesture has finished.
+    /// A mouse button has been let go.
     LOK_MOUSEEVENT_MOUSEBUTTONUP,
-    /// A change has happened during a press gesture.
+    /// The mouse has moved while a button is pressed.
     LOK_MOUSEEVENT_MOUSEMOVE
 }
 LibreOfficeKitMouseEventType;
